use strict;

package ISPAPI::Connection;

use LWP::UserAgent;

sub new {
	my $package = shift;
	my %config = @_;

	my $self = {%config};
	bless $self, $package;

	return $self;
}


sub query_list {
	my $self = shift;
	my $response = $self->call(@_);

	my $list = {
		CODE => $response->{CODE},
		DESCRIPTION => $response->{DESCRIPTION},
		RUNTIME => $response->{RUNTIME},
		QUEUETIME => $response->{QUEUETIME},
		ITEMS => []
	};

	if ( exists $response->{PROPERTY} ) {
		foreach my $property ( keys %{$response->{PROPERTY}} ) {
			if ( $property =~ /^(FIRST|LAST|COUNT|LIMIT|TOTAL)$/i ) {
				$list->{$property} = $response->{PROPERTY}{$property}[0];
			}
			else {
				my $index = 0;
				foreach my $value ( @{$response->{PROPERTY}{$property}} ) {
					$list->{ITEMS}[$index]{$property} = $value;
					$index++;
				}
			}
		}
	}

	if ( (exists $list->{FIRST}) && (exists $list->{LIMIT}) ) {
		$list->{PAGE} = int($list->{FIRST} / $list->{LIMIT}) + 1;
		if ( $list->{PAGE} > 1 ) {
			$list->{PREVPAGE} = $list->{PAGE} - 1;
			$list->{PREVPAGEFIRST} = ($list->{PREVPAGE} - 1) * $list->{LIMIT};
		}
		$list->{NEXTPAGE} = $list->{PAGE} + 1;
		$list->{NEXTPAGEFIRST} = ($list->{NEXTPAGE} - 1) * $list->{LIMIT};
	}

	if ( (exists $list->{TOTAL}) && (exists $list->{LIMIT}) ) {
		$list->{PAGES} = int(($list->{TOTAL} + $list->{LIMIT} - 1) / $list->{LIMIT});
		$list->{LASTPAGEFIRST} = ($list->{PAGES} - 1) * $list->{LIMIT};
		if ( (exists $list->{NEXTPAGE}) && ($list->{NEXTPAGE} > $list->{PAGES}) ) {
			delete $list->{NEXTPAGE};
			delete $list->{NEXTPAGEFIRST};
		}
	}
	
	return $list;
}


sub call {
	my $self = shift;
	return ISPAPI::parse_response($self->call_raw(@_));
}


sub call_raw {
	my $self = shift;
	return $self->call_raw_http(@_);
}


sub call_raw_http {
	my $self = shift;
	my $command = shift;
	my $config = shift;

	if ( !exists $self->{UserAgent} ) {
		$self->{UserAgent} = new LWP::UserAgent(
			agent => "ispapi-perl/1.0.0",
			keep_alive => 4
		);
	}

	my $url = "https://coreapi.1api.net/api/call.cgi";

	my $post = {
		s_command => ISPAPI::create_command($command)
	};
	$post->{s_entity} = $self->{Entity} if exists $self->{Entity};
	$post->{s_login} = $self->{Login} if exists $self->{Login};
	$post->{s_pw} = $self->{Password} if exists $self->{Password};

	my $response = $self->{UserAgent}->post($url, $post );
	return $response->content();	
}



package ISPAPI;

use strict;

sub connect {
	return new ISPAPI::Connection( @_ );
}

sub parse_response {
    my $response = shift;

    return () if !defined $response;
    
    my %hash;

    foreach ( split /\n/, $response ) {
        if ( /^([^\=]*[^\t\= ])[\t ]*=[\t ]*/ ) {
            my $attr = $1;
            my $value = $';
            $value =~ s/[\t ]*$//;
            if ( $attr =~ /^property\[([^\]]*)\]/i ) {
                my $prop = uc $1;
                $prop =~ s/\s//og;
                if ( exists $hash{"PROPERTY"}->{$prop} ) {
                    push @{$hash{"PROPERTY"}->{$prop}}, $value;
                }
                else {
                     $hash{"PROPERTY"}->{$prop} = [$value];
                }
            }
            else {
                $hash{uc $attr} = $value;
            }
        }
        elsif ( /:([^\n]*)/ ) {
            my $attr = $`;
            my $value = $1;
            if ( $attr =~ /^property\[([^\]]*)\]/i ) {
                my $prop = uc $1;
                $prop =~ s/\s//og;
                if ( exists $hash{"PROPERTY"}->{$prop} ) {
                    push @{$hash{"PROPERTY"}->{$prop}}, $value;
                }
                else {
                     $hash{"PROPERTY"}->{$prop} = [$value];
                }
            }
            else {
                $hash{uc $attr} = $value;
            }
        }
    }
    return \%hash;
	
}

sub create_command {

    my $hash = shift;

    my $command = "";
    my $k;
    foreach $k ( keys %{$hash} ) {
        if ( defined $hash->{$k} ) {
            if ( ref $hash->{$k} ) {
                my $i;
                for ( $i = 0; $i <= $#{$hash->{$k}}; $i++ ) {
                    if ( defined $hash->{$k}->[$i] ) {
                        my $v = $hash->{$k}->[$i];
                        $v =~ s/\r|\n//og;
                        $command .= "$k$i=$v\n";
                    }
                }
            }
            else {
                my $v = $hash->{$k};
                $v =~ s/\r|\n//og;
                $command .= "$k=$v\n";
            }
        }
    }
    return $command;
}


1;

